/*
htop
(C) 2004 Hisham H. Muhammad
Released under the GNU GPL, see the COPYING file
in the source distribution for its full text.
*/

#include "Process.h"
#include "ProcessFilter.h"
#include "Object.h"

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

// TODO: wtf!?
int kill(pid_t pid, int signal);

// TODO: can this be really *hardcoded*!?
#define PAGE_SIZE 4

#define PROCESS_COMM_LEN 512
#define PROCESS_USER_LEN 10

/*{
typedef struct Process_ {
   Object super;

   ProcessFilter* filter;
   char* displayBuffer;

   int pid;
   char comm[PROCESS_COMM_LEN + 2];
   char state;
   int ppid;
   int pgrp;
   int session;
   int tty_nr;
   int tpgid;
   unsigned long int flags;
   unsigned long int minflt;
   unsigned long int cminflt;
   unsigned long int majflt;
   unsigned long int cmajflt;
   unsigned long int utime;
   unsigned long int stime;
   long int cutime;
   long int cstime;
   long int priority;
   long int nice;
   long int itrealvalue;
   unsigned long int starttime;
   unsigned long int vsize;
   long int rss;
   unsigned long int rlim;
   unsigned long int startcode;
   unsigned long int endcode;
   unsigned long int startstack;
   unsigned long int kstkesp;
   unsigned long int kstkeip;
   unsigned long int signal;
   unsigned long int blocked;
   unsigned long int sigignore;
   unsigned long int sigcatch;
   unsigned long int wchan;
   unsigned long int nswap;
   unsigned long int cnswap;
   int exit_signal;
   int processor;
   int m_size;
   int m_resident;
   int m_share;
   int m_trs;
   int m_drs;
   int m_lrs;
   int m_dt;
   uid_t st_uid;
   float percent_cpu;
   char user[PROCESS_USER_LEN + 1];
} Process;
}*/

/* private property */
char* PROCESS_CLASS = "Process";

Process* Process_clone(Process* this) {
   Process* clone = malloc(sizeof(Process));
   memcpy(clone, this, sizeof(Process));
   return clone;
}

Process* Process_new(ProcessFilter* filter) {
   Process* this = malloc(sizeof(Process));
   ((Object*)this)->class = PROCESS_CLASS;
   ((Object*)this)->display = Process_display;
   this->filter = filter;
   this->displayBuffer = malloc(filter->width + 1);
   return this;
}

void Process_delete(Process* this) {
   free(this->displayBuffer);
   free(this);
}

char* Process_display(Object* cast) {
   Process* this = (Process*) cast;
   ProcessField* fields = this->filter->fields;
   char* line = this->displayBuffer;
   char* writeLine = line;
   int remaining = this->filter->width;
   for (int i = 0; fields[i] != LAST; i++) {
      int walked = Process_snprintf(this, writeLine, remaining, fields[i]);
      writeLine += walked;
      remaining -= walked;
      if (!remaining) break;
   }
   return line;
}

void Process_setPriority(Process* this, int priority) {
   int err = setpriority(PRIO_PROCESS, this->pid, priority);
   if (err == 0) {
      this->nice = priority;
   }
}

void Process_sendSignal(Process* this, int signal) {
   kill(this->pid, signal);
}

int Process_snprintf(Process* this, char* out, int n, ProcessField field) {
   switch (field) {
   case PID: snprintf(out, n, "%5d ", this->pid); return MIN(6, n);
   case COMM: snprintf(out, n, "%s ", this->comm); return MIN(PROCESS_COMM_LEN, n);
   case STATE: snprintf(out, n, "%c ", this->state); return MIN(2, n);
   case PRIORITY: snprintf(out, n, "%3ld ", this->priority); return MIN(4, n);
   case NICE: snprintf(out, n, "%3ld ", this->nice); return MIN(4, n);
   case M_SIZE: snprintf(out, n, "%5d ", this->m_size * PAGE_SIZE); return MIN(6, n);
   case M_RESIDENT: snprintf(out, n, "%5d ", this->m_resident * PAGE_SIZE); return MIN(6, n);
   case M_SHARE: snprintf(out, n, "%5d ", this->m_share * PAGE_SIZE); return MIN(6, n);
   case ST_UID: snprintf(out, n, "%4d ", this->st_uid); return MIN(5, n);
   case USER: snprintf(out, n, "%-8s ", this->user); return MIN(9, n);
   case UTIME: snprintf(out, n, "%5ld ", this->utime); return MIN(6, n);
   case STIME: snprintf(out, n, "%5ld ", this->stime); return MIN(6, n);
   case PERCENT_CPU: {
      if (this->percent_cpu > 99.9) {
         snprintf(out, n, "100. "); 
      } else {
         snprintf(out, n, "%4.1f ", this->percent_cpu); return MIN(5, n);
      }
      return MIN(5, n);
   }
   default:
      snprintf(out, n, "- "); return MIN(2, n);
   }
   return 0;
}

int Process_compare(const void* v1, const void* v2) {
   Process* p1 = (Process*)v1;
   Process* p2 = (Process*)v2;
   switch (p1->filter->sortKey) {
   case PID:
      return p2->pid - p1->pid;
   case PERCENT_CPU:
      return p2->percent_cpu - p1->percent_cpu;
   default:
      return p2->pid - p1->pid;
   }
}
