/*
htop
(C) 2004 Hisham H. Muhammad
Released under the GNU GPL, see the COPYING file
in the source distribution for its full text.
*/

#include "Object.h"
#include "Process.h"
#include "ProcessFilter.h"
#include <stdlib.h>
#include <string.h>

/*{
typedef enum ProcessField_ {
   PID, COMM, STATE, PPID, PGRP, SESSION, TTY_NR, TPGID, FLAGS, MINFLT, CMINFLT, MAJFLT, CMAJFLT, UTIME,
   STIME, CUTIME, CSTIME, PRIORITY, NICE, ITREALVALUE, STARTTIME, VSIZE, RSS, RLIM, STARTCODE, ENDCODE,
   STARTSTACK, KSTKESP, KSTKEIP, SIGNAL, BLOCKED, SIGIGNORE, SIGCATCH, WCHAN, NSWAP, CNSWAP, EXIT_SIGNAL,
   PROCESSOR, M_SIZE, M_RESIDENT, M_SHARE, M_TRS, M_DRS, M_LRS, M_DT, ST_UID, PERCENT_CPU, USER, LAST
} ProcessField;

typedef struct ProcessFilter_ {
   Object super;

   ProcessField* fields;
   int width;
   ProcessField sortKey;
} ProcessFilter;
}*/

/* private property */
char* PROCESSFILTER_CLASS = "ProcessField";

/* private property */
ProcessField hardcodedFields[] = { PID, USER, PRIORITY, NICE, M_SIZE, M_RESIDENT, M_SHARE, STATE, PERCENT_CPU, COMM, LAST };

ProcessFilter* ProcessFilter_new() {
   ProcessFilter* this = malloc(sizeof(ProcessFilter));
   ((Object*)this)->class = PROCESSFILTER_CLASS;
   ((Object*)this)->display = ProcessFilter_display;
   this->fields = hardcodedFields;
   this->width = 512;
   this->sortKey = PERCENT_CPU;
   return this;
}

char* ProcessFilter_display(Object* cast) {
   ProcessFilter* this = (ProcessFilter*) cast;
   ProcessField* fields = this->fields;
   char* line = malloc(this->width + 1);
   char* writeLine = line;
   int remaining = this->width;
   for (int i = 0; fields[i] != LAST; i++) {
      char* header = ProcessFilter_printHeader(this, fields[i]);
      strncpy(writeLine, header, remaining);
      int walked = strlen(header);
      writeLine += walked;
      remaining -= walked;
      if (!remaining) break;
   }
   return line;
}

char* ProcessFilter_printHeader(ProcessFilter* this, ProcessField field) {
   switch (field) {
   case PID: return "  PID ";
   case COMM: return "COMM";
   case STATE: return "S ";
   case PRIORITY: return " PR ";
   case NICE: return " NI ";
   case M_SIZE: return " VIRT ";
   case M_RESIDENT: return "  RES ";
   case M_SHARE: return "  SHR ";
   case ST_UID: return " UID ";
   case USER: return "USER     ";
   case UTIME: return " UT   ";
   case STIME: return " ST   ";
   case PERCENT_CPU: return "CPU% ";
   default: return "- ";
   }
}
