/*
htop
(C) 2004 Hisham H. Muhammad
Released under the GNU GPL, see the COPYING file
in the source distribution for its full text.
*/

#include "Vector.h"
#include <stdlib.h>
#include <string.h>

/*{
typedef void(*Procedure)(void*);
typedef int(*BooleanFunction)(const void*,const void*);

typedef struct Vector_ {
   void **array;
   int arraySize;
   int growthRate;
   int items;
   BooleanFunction compareFunction;
} Vector;

}*/

Vector* Vector_new() {
   Vector* v;
   int arraySize;

   arraySize = 10;
   v = (Vector*) malloc(sizeof(Vector));
   v->growthRate = arraySize;
   v->array = (void**) calloc(arraySize, sizeof(void*));
   v->arraySize = arraySize;
   v->compareFunction = Vector_compareFunction;
   v->items = 0;
   return v;
}

void Vector_delete(Vector* v) {
   int i;

   for (i = 0; i < v->items; i++)
      if (v->array[i])
         free(v->array[i]);
   free(v->array);
   free(v);
}

void Vector_prune(Vector* v) {
   int i;

   for (i = 0; i < v->items; i++)
      if (v->array[i])
         free(v->array[i]);
   v->items = 0;
}

Vector* Vector_clone(Vector* v) {
   Vector* clone;

   clone = (Vector*) malloc(sizeof(Vector));
   clone->growthRate = v->growthRate;
   clone->arraySize = v->arraySize;
   clone->array = (void**) malloc(v->arraySize * sizeof(void*));
   memcpy(clone->array, v->array, v->arraySize * sizeof(void*));
   clone->items = v->items;
   return clone;
}

int Vector_compareFunction(const void* v1, const void* v2) {
   return !(v1 == v2);
}

void Vector_setCompareFunction(Vector* v, BooleanFunction f) {
   v->compareFunction = f;
}

void Vector_sort(Vector* v) {
   for (int i = 0; i < v->items; i++) {
      for (int j = i+1; j < v->items; j++) {
         if (v->compareFunction(v->array[i], v->array[j]) > 0) {
            void* tmp = v->array[i];
            v->array[i] = v->array[j];
            v->array[j] = tmp;
         }
      }
   }
}

/* private */
void Vector_checkArraySize(Vector* this) {
   if (this->items >= this->arraySize) {
      int i;
      i = this->arraySize;
      this->arraySize = this->items + this->growthRate;
      this->array = (void**) realloc(this->array, sizeof(void*) * this->arraySize);
      for (; i < this->arraySize; i++)
         this->array[i] = NULL;
   }
}

void Vector_insert(Vector* this, int index, void* data) {
   Vector_checkArraySize(this);
   for (int i = this->items; i >= index; i--) {
      this->array[i+1] = this->array[i];
   }
   this->array[index] = data;
   this->items++;
}

void* Vector_remove(Vector* this, int index) {
   void* removed = this->array[index];
   int subsequent = this->items - index - 1;
   if (subsequent > 0)
      memmove(this->array[index], this->array[index+1], subsequent * sizeof(void*));
   this->items--;
   this->array[this->items] = NULL;
   return removed;
}

void Vector_set(Vector* v, int index, void* data) {
   Vector_checkArraySize(v);
   if (index >= v->items) {
      v->items = index+1;
   }
   v->array[index] = data;
}

void* Vector_get(Vector* v, int index) {
   if (index >= v->items)
      return NULL;
   return v->array[index];
}

int Vector_size(Vector* v) {
   return v->items;
}

void Vector_merge(Vector* v, Vector* v2) {
   int i;
   
   for (i = 0; i < v2->items; i++)
      Vector_add(v, v2->array[i]);
   v2->items = 0;
   Vector_delete(v2);
}

void Vector_add(Vector* v, void* data) {
   Vector_set(v, v->items, data);
}

int Vector_indexOf(Vector* v, void* search) {
   int i;

   for (i = 0; i < v->items; i++)
      if (v->compareFunction(v->array[i], search) == 0)
         return i;
   return -1;
}

void Vector_foreach(Vector* v, Procedure f) {
   int i;

   for (i = 0; i < v->items; i++)
      f(v->array[i]);
}
